Option Explicit

' How many retries to correct the error
' If -1 then retries will go on forever.
' Retries counter (Value > 0) must only be used when there
' are line numbers in your program.  Otherwise the routine will not be able to
' reset the counter for each new line.
Global ErrHndlRetryCount%
' How long to wait for before retrying (Seconds)
Global ErrHndlRetryTime#
' Should the resume next option be there
Global ErrHndlResumeNext%
Global ErrHndlPlaySound%
Global ErrHndlSoundFile$
' Name of the Jet Error
Global ErrHndlJetErrorFile$
'%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
' Following global variable is used only by the error handler.
Global ErrHndlRet%
Dim CurrRetryCnt%
Dim KeepTrying%


Type JetErrors
    ErrNo As String * 10
    ErrMsg As String * 70
    CRLF As String * 2
End Type

Const ATTR_NORMAL = 0

Function ErrHndlr% (ByVal ProcName$, ByVal ErrNo%, ByVal ErrLineNo%, ErrMsg$)
    Static PrevLine&, PrevProc$, SMsg$
    If PrevProc = "" Then
        ' Initialize the line number. Only once during the life cycle of the program.
        PrevLine = -1
    End If
    If (PrevProc <> ProcName) Or (PrevLine <> ErrLineNo) Then
        ' This will get initialized only once for a statement.
        CurrRetryCnt = ErrHndlRetryCount
        PrevProc = ProcName
        PrevLine = ErrLineNo
        KeepTrying = 0
    End If
    ErrHndlRet = 0
    ErrMsg = Error(ErrNo)
    If InStr(1, UCase(ErrMsg), "RESERVED ERROR") > 0 Then
        SMsg = ErrHndlrGetJetError(ErrNo)
        If SMsg <> "" Then
            ErrMsg = SMsg
        End If
    End If
    ErrMsg = "Error (" & ErrNo & "-" & ErrMsg & ") occurred in " & ProcName
    If ErrLineNo > 0 Then
        ErrMsg = ErrMsg & " @ Line# " & Trim(Str(ErrLineNo))
    End If
    Load frmErrHndlr
    If ErrHndlResumeNext Then
        frmErrHndlr!pbResumeNext.Enabled = True
    Else
        frmErrHndlr!pbResumeNext.Enabled = False
    End If
    If ErrHndlRetryCount = -1 Then
        frmErrHndlr!ckKeepTrying = "1"
        frmErrHndlr!ckKeepTrying.Enabled = False
    Else
        frmErrHndlr!ckKeepTrying.Enabled = True
        If CurrRetryCnt > 0 Then
            frmErrHndlr!lblCountDown = Trim(Str(CurrRetryCnt))
        End If
    End If
    If (ErrHndlRetryTime > 0) And (ErrHndlRetryCount <> 0) Then
        frmErrHndlr!tmrRetry.Interval = ErrHndlRetryTime * 1000
    Else
        frmErrHndlr!tmrRetry.Interval = 0
    End If

    frmErrHndlr!ckPlaySound = ErrHndlPlaySound
    frmErrHndlr!ckKeepTrying = KeepTrying
    frmErrHndlr!lblCountDown = Trim(Str(CurrRetryCnt))
    frmErrHndlr!txtMsg = ErrMsg
    frmErrHndlr.Show 1
    DoEvents
    ErrHndlr = ErrHndlRet
End Function

Sub ErrHndlrAdj1 (pVal$)
    ' For internal use only
    ' This sub updates module wide variable KeepTrying with the check box ckKeepTrying on the form.
    KeepTrying = Val(pVal)
End Sub

Function ErrHndlrGetJetError$ (ByVal ErrNo&)
    ' For internal use only
    Dim fNum%, Msg$, JetRec As JetErrors
    Static JetFileNotFound%
    Msg = ""
    If Dir$(ErrHndlJetErrorFile, ATTR_NORMAL) = "" Then
        ' Jet File Not found
        If Not JetFileNotFound Then
            JetFileNotFound = True
            MsgBox ("Jet Error file not found. Call support.")
        End If
        GoTo ErrHndlrGetJetError_Skip1
    Else
        JetFileNotFound = False
    End If
    fNum = FreeFile
    Open ErrHndlJetErrorFile For Random As #fNum Len = 82
    Do While True
        Get #fNum, , JetRec
        If EOF(fNum) Then
            Exit Do
        End If
        If Val(Trim(JetRec.ErrNo)) = ErrNo Then
            Msg = "Jet Error- " & JetRec.ErrMsg
            Exit Do
        End If
    Loop
    Close #fNum
ErrHndlrGetJetError_Skip1:
    ErrHndlrGetJetError = Msg
End Function

Function ErrHndlrRetry& (pInCnt&)
    ' For internal use only
    ' This function updates the retry counter within this module from the form module
    CurrRetryCnt = pInCnt - 1
    ErrHndlrRetry = CurrRetryCnt
End Function

