Option Explicit

'***********************************************************************
' Description:
'   Contains function declarations, constants, and user-defined
'   types for the FileNet WAL API.
'
'***********************************************************************
'      CHANGE LOG (most recent at top)
' Name          Date     Description
' Ulmer         09/10/93 New
'***********************************************************************

'***********************************************************************
'***********************************************************************
' USER DEFINED TYPES
'***********************************************************************
'***********************************************************************

Type typNCHDomainName
    sOrganization As String * 20
    sDomain As String * 20
End Type

Type typNCHObjectName
    sOrganization As String * 20
    sDomain As String * 20
    sObject As String * 40
End Type

Type typNCHDefServiceDescValue
    nLevel As Integer
    DefaultBES As String * 80
    DefaultDtarsService As String * 80
    DefaultIMS As String * 80
    DefaultPrintService As String * 80
    DefaultSKFService As String * 80
    DefaultWFLQService As String * 80
    DefaultSortService As String * 80
    DefaultSQLService As String * 80
End Type


'***********************************************************************
'***********************************************************************
' WAL API FUNCTIONS
'***********************************************************************
'***********************************************************************

Declare Sub ERM_get_error Lib "erm.dll" (ByVal lError As Long, ByVal bServerFlag As Integer, ByVal lpErrorText As String)
Declare Sub IAFLIB_get_user_name Lib "iaflib.dll" (ByVal lpUserName As String)
Declare Function IAFLIB_logon_user Lib "iaflib.dll" (ByVal lpUser As Any, ByVal lpPassword As Any, lpIMSName As Any, lpPrintServiceName As Any) As Long
Declare Function RDD_get_handle Lib "rdd.dll" (ByVal lpHandleName As String) As Integer
Declare Function RDD_set_handle Lib "rdd.dll" (ByVal lpHandleName As String, ByVal hWnd As Integer) As Long
Declare Function ServiceNameDefaults Lib "servname.dll" (DomainName As typNCHDomainName, NCHDefServiceDescValue As typNCHDefServiceDescValue) As Long

Sub CloseFileNetDisplay (ByVal sWindow As String)
'***********************************************************************
' Description:
'   Closes the specified FileNet Image Display windows.
'
' Arguments:
'   sWindow
'       A string containing the names of one or move FileNet
'       Image Display windows.  The windows are named
'       A, B, C, and R, so the string can contain any combination
'       of those letters
'       Modified: No
'
'***********************************************************************
'      CHANGE LOG (most recent at top)
' Name          Date     Description
' Ulmer         11/11/93 New
'***********************************************************************

    Dim hWnd As Integer
    Dim i As Integer
    Dim nRC As Integer
    
    sWindow = UCase$(Trim$(sWindow))

    For i = 1 To Len(sWindow)
        hWnd = FindWindowByName(0&, "Image Display " & Mid$(sWindow, i, 1))
        If hWnd <> 0 Then
            nRC = PostMessage(hWnd, WM_CLOSE, 0, 0&)
        End If
    Next i

End Sub

Sub DisplayDocs (mhict As Control, lbl As Control, ByVal sWindow As String)
'***********************************************************************
' Description:
'   Displays selected documents in the requested image display window
'
' Arguments:
'   sWindow
'       The display window to open
'       Modified: No
'
'***********************************************************************
'      CHANGE LOG (most recent at top)
' Name          Date     Description
' Ulmer         10/13/93 New
'***********************************************************************
    
    Dim anSelItems() As Integer
    Dim alDocID() As Long
    Dim i As Integer
    Dim nItems As Integer
    Dim sMsg As String

    '   set hourglass cursor
    Screen.MousePointer = HOURGLASS

    '   get selected documents
    GetMultiListSelectedItems mhict.hWnd, nItems, anSelItems()

    '   at least 1 document must be selected
    If nItems = 0 Then
        sMsg = "Please select one or more documents to display"
        Screen.MousePointer = DEFAULT
        MsgBox sMsg, MB_OK + MB_ICONEXCLAMATION, "No Documents Selected"
        mhict.SetFocus
        Exit Sub
    End If

    '   build array of documents to display
    ReDim alDocID(nItems - 1)
    For i = 0 To nItems - 1
        mhict.GetItem(anSelItems(i)) = True
        alDocID(i) = CLng(mhict.CellText(4))
    Next i

    ' call filenet display interface to display documents selected
    FileNetDisplayInterface lbl, alDocID(), sWindow

    'unselect all documents
    SetMultiListDeselectAll mhict

    '   set arrow cursor
    Screen.MousePointer = DEFAULT

End Sub

Sub FileNetDisplayInterface (lbl As Control, alDocID() As Long, ByVal sWindow As String)
'***********************************************************************
' Description:
'   Uses DDE to display documents

' Arguments:
'   lbl
'       A label control which is used to contain the
'       data to be POKEd to image display.
'       Modified: Yes
'
'   alDocID()
'       array of documents to display
'       Modified: No
'
'   sWindow
'       display window to view the documents
'       Modified: No
'
' Global Variables:
'   gsUserNameFirst
'       Operator's first name
'       Modified: No
'
'   gsUserNameLast
'       Operator's last name
'       Modified: No
'
'   gsMDIGateway
'       MDI Gateway used to connect to DB2
'       Modified: No
'
'***********************************************************************
'      CHANGE LOG (most recent at top)
' Name          Date     Description
' Nystrom, J.   02/14/94 Increase the timeout parameter to 60 seconds.
' Ulmer         01/25/94 Fixed problem relating to DDE timeout error
' Carolus       09/20/93 New
'***********************************************************************

    Dim ErrorInfo As typErrorInfo
    Dim hTask As Integer
    Dim i As Integer
    Dim sDocs As String
    Dim sMsg As String

    '   set up error handler
    On Error GoTo FileNetDisplayInterfaceError

    '   Build data to poke to image display.  It's a string in the
    '   format "doc# tab startpage tab endpage".  For all pages, use
    '   0 for the endpage.  Multiple "doc# tab startpage tab endpage"
    '   sequences may be specified, separated by tabs
    sDocs = ""
    For i = 0 To UBound(alDocID)
        If i > 0 Then
            sDocs = sDocs & Chr$(9)
        End If
        sDocs = sDocs & Format$(alDocID(i)) & Chr$(9) & "1" & Chr$(9) & "0"
    Next i

    '   poke the data to image display
    lbl.LinkMode = gDDE_LINK_NONE
    lbl.LinkTimeout = 600   '   set 60 second timeout   JN021494
    lbl.LinkTopic = "FileNet Display " & UCase$(Trim$(sWindow)) & "|" & "System"
    lbl.LinkMode = gDDE_LINK_MANUAL
    lbl.Caption = sDocs
    lbl.LinkItem = "PageRangeList"
    lbl.LinkPoke
    lbl.LinkMode = gDDE_LINK_NONE

    Exit Sub
   
FileNetDisplayInterfaceError:
    '   if the image display window isn't running, then open it
    If Err = gDDE_NO_APP_RESPONDED Then
        hTask = Shell("dispimg.exe 0 0 " & UCase$(Trim$(sWindow)), 1)
        DoEvents
        Resume  '   try again
    ElseIf Err = gDDE_TIMEOUT Then                      '   MU012594
        lbl.Caption = ""                                '   MU012594
    Else
        '   get error message from image display
        lbl.LinkItem = "ReturnMessage"
        lbl.LinkRequest
        If Left$(lbl.Caption, 7) = "<0,0,0>" Then   '   false error
            Resume Next
        End If
    End If
    '   display error message
    Screen.MousePointer = DEFAULT
    sMsg = "Unable to display selected documents in display window " & UCase$(Trim$(sWindow))
    MsgBox sMsg, MB_OK + MB_ICONSTOP, "DDE Error"
    '   initialize error info
    ErrorInfo.sErrorMessage = Error$ & gsCRLF & lbl.Caption
    ErrorInfo.vErrorCode = Err
    ErrorInfo.sErrorFunction = "L"
    ErrorInfo.sUserName = gsUserNameFirst & " " & gsUserNameLast
    ErrorInfo.sProcedureName = "FileNetDisplayInterface"
    ErrorInfo.sMDIGateway = gsMDIGateway
    '   log error
    LogError ErrorInfo
    '   that's all
    Exit Sub

End Sub

Sub FileNetLogoff ()
'***********************************************************************
' Description:
'   Uses FileNet WAL APIs to logoff FileNet
'
'***********************************************************************
'      CHANGE LOG (most recent at top)
' Name          Date     Description
' Ulmer         09/14/93 New
'***********************************************************************
    
    Dim hLTS As Integer
    Dim nFNLogoffMsgID As Integer
    Dim lRC As Long
    Dim nRC As Integer

    '   register broadcase message for other FN apps
    nFNLogoffMsgID = RegisterWindowMessage("FN_logoff")

    '   send broadcase message to other FN apps
    lRC = SendMessage(HWND_BROADCAST, nFNLogoffMsgID, 0, 0&)
    
    '   get handle to LTS task
    hLTS = RDD_get_handle("LTS")
    
    '   shut down LTS
    If hLTS <> 0 Then
        nRC = PostMessage(hLTS, WM_CLOSE, 0, 0&)
    End If
    
    '   logoff
    lRC = IAFLIB_logon_user(ByVal 0&, ByVal 0&, ByVal 0&, ByVal 0&)

End Sub

Function sFileNetLogon (ByVal sUserID As String, ByVal sPassword As String, ByVal hWnd As Integer) As String
'***********************************************************************
' Description:
'   Uses FileNet WAL APIs to logon to FileNet
'
' Arguments:
'   sUserID
'       The FileNet User ID
'       Modified: No
'
'   sPassword
'       The FileNet password
'       Modified: No
'
'   hWnd
'       The window handle of the logon window
'       Modified: No
'
' Returns:
'   An empty string if the logon is successful, otherwise an error message.
'
'***********************************************************************
'      CHANGE LOG (most recent at top)
' Name          Date     Description
' Ulmer         09/10/93 New
'***********************************************************************

    Dim DefaultDomain As typNCHDomainName
    Dim DefaultServices As typNCHDefServiceDescValue
    Dim DefaultIMS As typNCHObjectName
    Dim dtCurrent As Variant
    Dim dtEnd As Variant
    Dim lRC As Long
    Dim nFNBaseMsgID As Integer
    Dim nRC As Integer
    Dim sUserName As String * 255
    Dim sValue As String * 255

    '   register windows message used to communicate with FileNet LTS
    nFNBaseMsgID = RegisterWindowMessage("FN_Base_Msg")

    '   register the logon window handle in FileNet Remote Data Dictionary
    lRC = RDD_set_handle("LOGON", hWnd)

    '   launch Logon Transport Services
    nRC = WinExec("lts.exe", SW_HIDE)

    '   if unable to launch, then set return message and exit
    If nRC >= 0 And nRC < 32 Then
        sFileNetLogon = "FileNet Login Error - Can't launch LTS.  Error code: " & Str$(nRC)
        Exit Function
    End If

    '   wait a few seconds so we're sure LTS is finished initializing
    dtCurrent = Now
    dtEnd = DateAdd("s", 3, dtCurrent)
    Do While dtCurrent <= dtEnd
        DoEvents
        dtCurrent = Now
    Loop

    '   get the name of the default domain
    nRC = GetPrivateProfileString("PCWS", "Domain", "", sValue, 255, "c:\filenet\desktop\logon.cfg")
    If nRC < 1 Then
        sFileNetLogon = "FileNet Login Error - Unable to retrieve default domain name from logon.cfg."
        Exit Function
    End If
    DefaultDomain.sDomain = Left$(sValue, nRC) & String$(20 - nRC, 0)
    nRC = GetPrivateProfileString("PCWS", "Organization", "", sValue, 255, "c:\filenet\desktop\logon.cfg")
    If nRC < 1 Then
        sFileNetLogon = "FileNet Login Error - Unable to retrieve default domain name from logon.cfg."
        Exit Function
    End If
    DefaultDomain.sOrganization = Left$(sValue, nRC) & String$(20 - nRC, 0)

    '   get name of the default IMS
    lRC = ServiceNameDefaults(DefaultDomain, DefaultServices)
    If lRC <> 0 Then
        sFileNetLogon = "FileNet Login Error - " & sGetFNErrorText(lRC)
        Exit Function
    End If

    '   set up default IMS to pass to logon
    DefaultIMS.sOrganization = sPadStringWithNull(Left$(DefaultServices.DefaultIMS, 20))
    DefaultIMS.sDomain = sPadStringWithNull(Mid$(DefaultServices.DefaultIMS, 21, 20))
    DefaultIMS.sObject = sPadStringWithNull(Right$(DefaultServices.DefaultIMS, 40))

    '   logon.  Convert UserID to lower case, since FileNet IDs are case
    '   sensitive, but should be defined using all lower case.
    lRC = IAFLIB_logon_user(LCase$(Trim$(sUserID)), Trim$(sPassword), DefaultIMS, ByVal 0&)
    If lRC <> 0 Then
        sFileNetLogon = "FileNet Login Error - " & sGetFNErrorText(lRC)
        Exit Function
    End If
    
    '   unregister the logon window handle
    lRC = RDD_set_handle("LOGON", 0)

    '   verify that the logon was successful by retrieving the user ID
    '   (FileNet calls this the user name, but it's really the user ID)
    IAFLIB_get_user_name sUserName
    If Left$(sUserName, 1) = ":" Then
        sFileNetLogon = "FileNet Login Error - Unable to login"
        Exit Function
    End If

    '   if we got here, the logon was successful, so return an
    '   empty string to indicate success
    sFileNetLogon = ""

End Function

Private Function sGetFNErrorText (ByVal lError As Long) As String
'***********************************************************************
' Description:
'   Retrieves the FileNet error tuple and associated error text.
'
' Arguments:
'   lError
'       A FileNet error code
'       Modified: No
'
' Returns:
'   The error tuple and text associated with lError
'
'***********************************************************************
'      CHANGE LOG (most recent at top)
' Name          Date     Description
' Ulmer         09/10/93 New
'***********************************************************************

    Dim sErrorText As String * 255

    '   get the error text
    ERM_get_error lError, False, sErrorText

    '   fix up ASCIIZ string
    sGetFNErrorText = sTruncateStringAtNull(sErrorText)

End Function

